exportPattern("^[^\\.]")

import(stats)
import(utils)
import(graphics)
import(grDevices)
import(methods)
import(Matrix)
importFrom(nlme, ranef)
export(ranef)
import(mathjaxr)
import(metadat)
import(numDeriv)

S3method("[", list.rma)
S3method("$<-", list.rma)
S3method("[", escalc)
S3method("$<-", escalc)
S3method(addpoly, default)
S3method(addpoly, predict.rma)
S3method(addpoly, rma)
S3method(aggregate, escalc)
S3method(AIC, rma)
S3method(anova, rma)
S3method(as.data.frame, anova.rma)
S3method(as.data.frame, confint.rma)
S3method(as.data.frame, vif.rma)
S3method(as.data.frame, list.anova.rma)
S3method(as.data.frame, list.confint.rma)
S3method(as.data.frame, list.rma)
S3method(as.matrix, list.rma)
S3method(baujat, rma)
S3method(BIC, rma)
S3method(blup, rma.uni)
S3method(cbind, escalc)
S3method(coef, matreg)
S3method(coef, rma)
S3method(coef, summary.rma)
S3method(coef, permutest.rma.uni)
S3method(confint, rma.glmm)
S3method(confint, rma.mh)
S3method(confint, rma.mv)
S3method(confint, rma.peto)
S3method(confint, rma.uni)
S3method(confint, rma.uni.selmodel)
S3method(confint, rma.ls)
S3method(cooks.distance, rma.mv)
S3method(cooks.distance, rma.uni)
S3method(cumul, rma.mh)
S3method(cumul, rma.peto)
S3method(cumul, rma.uni)
S3method(deviance, rma)
S3method(df.residual, rma)
S3method(dfbetas, rma.mv)
S3method(dfbetas, rma.uni)
S3method(fitstats, rma)
S3method(fitted, rma)
S3method(forest, default)
S3method(forest, rma)
S3method(forest, cumul.rma)
S3method(formula, rma)
S3method(funnel, default)
S3method(funnel, rma)
S3method(gosh, rma)
S3method(hatvalues, rma.mv)
S3method(hatvalues, rma.uni)
S3method(hc, rma.uni)
S3method(influence, rma.uni)
S3method(labbe, rma)
S3method(leave1out, rma.mh)
S3method(leave1out, rma.peto)
S3method(leave1out, rma.uni)
S3method(logLik, rma)
S3method(regplot, rma)
S3method(model.matrix, rma)
S3method(nobs, rma)
S3method(permutest, rma.uni)
S3method(permutest, rma.ls)
S3method(plot, cumul.rma)
S3method(plot, gosh.rma)
S3method(plot, infl.rma.uni)
S3method(plot, permutest.rma.uni)
S3method(plot, profile.rma)
S3method(plot, vif.rma)
S3method(plot, rma.glmm)
S3method(plot, rma.mh)
S3method(plot, rma.peto)
S3method(plot, rma.uni)
S3method(plot, rma.uni.selmodel)
S3method(points, regplot)
S3method(predict, rma)
S3method(predict, rma.ls)
S3method(print, anova.rma)
S3method(print, confint.rma)
S3method(print, vif.rma)
S3method(print, list.anova.rma)
S3method(print, list.confint.rma)
S3method(print, escalc)
S3method(print, fsn)
S3method(print, gosh.rma)
S3method(print, infl.rma.uni)
S3method(print, list.rma)
S3method(head, list.rma)
S3method(tail, list.rma)
S3method(print, hc.rma.uni)
S3method(print, matreg)
S3method(print, permutest.rma.uni)
S3method(print, profile.rma)
S3method(print, ranktest)
S3method(print, regtest)
S3method(print, rma.glmm)
S3method(print, rma.mh)
S3method(print, rma.mv)
S3method(print, rma.peto)
S3method(print, rma.uni)
S3method(print, summary.matreg)
S3method(print, summary.rma)
S3method(print, tes)
S3method(profile, rma.mv)
S3method(profile, rma.uni)
S3method(profile, rma.uni.selmodel)
S3method(profile, rma.ls)
S3method(qqnorm, rma.glmm)
S3method(qqnorm, rma.mh)
S3method(qqnorm, rma.mv)
S3method(qqnorm, rma.peto)
S3method(qqnorm, rma.uni)
S3method(radial, rma)
S3method(ranef, rma.mv)
S3method(ranef, rma.uni)
S3method(rbind, escalc)
S3method(reporter, rma.uni)
S3method(residuals, rma)
S3method(robust, rma.mv)
S3method(robust, rma.uni)
S3method(selmodel, rma.uni)
S3method(rstandard, rma.mh)
S3method(rstandard, rma.mv)
S3method(rstandard, rma.peto)
S3method(rstandard, rma.uni)
S3method(rstudent, rma.mh)
S3method(rstudent, rma.mv)
S3method(rstudent, rma.peto)
S3method(rstudent, rma.uni)
S3method(simulate, rma)
S3method(summary, escalc)
S3method(summary, matreg)
S3method(summary, rma)
#S3method(traceplot, rma.uni)
S3method(trimfill, rma.uni)
S3method(update, rma)
S3method(vif, rma)
S3method(vcov, matreg)
S3method(vcov, rma)
S3method(weights, rma.glmm)
S3method(weights, rma.mh)
S3method(weights, rma.mv)
S3method(weights, rma.peto)
S3method(weights, rma.uni)
